﻿using Hotline.Application.Snapshot;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Snapshot;
using Microsoft.AspNetCore.Mvc;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Tools;
using Hotline.Snapshot.Interfaces;
using Hotline.Caching.Interfaces;
using XF.Utility.EnumExtensions;
using Hotline.Share.Enums.Snapshot;
using Quartz.Impl.Triggers;

namespace Hotline.Api.Controllers.Snapshot;

public class RedPackController : BaseController
{
    private readonly IRedPackApplication _redPackApplication;
    private readonly IIndustryRepository _industryRepository;
    private readonly ISystemDicDataCacheManager _sysDic;
    private readonly ISnapshotSMSTemplateRepository _snapshotSMSTemplateRepository;

    public RedPackController(IRedPackApplication redPackApplication, IIndustryRepository industryRepository, ISystemDicDataCacheManager sysDic, ISnapshotSMSTemplateRepository snapshotSMSTemplateRepository)
    {
        _redPackApplication = redPackApplication;
        _industryRepository = industryRepository;
        _sysDic = sysDic;
        _snapshotSMSTemplateRepository = snapshotSMSTemplateRepository;
    }

    #region 红包审核
    /// <summary>
    /// 获取市民红包审批列表
    /// </summary>
    /// <returns></returns>
    [HttpGet("audit")]
    public async Task<PagedDto<SnapshotOrderAuditItemsOutDto>> GetRedPackAuditItemsAsync([FromQuery] SnapshotOrderAuditItemsInDto dto)
        => (await _redPackApplication.GetRedPackAuditItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 获取网格员红包审批集合(政法委或应急管理局:根据账号登录返回数据)
    /// </summary>
    /// <returns></returns>
    [HttpGet("audit/guider")]
    public async Task<PagedDto<SnapshotOrderGuiderAuditItemsOutDto>> GetRedPackGuiderAuditItemsAsync([FromQuery] SnapshotOrderGuiderAuditItemsInDto dto)
        => (await _redPackApplication.GetRedPackGuiderAuditItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 获取网格员审核详情
    /// </summary>
    /// <param name="id">工单Id</param>
    /// <returns></returns>
    [HttpGet("audit/guider/{id}")]
    public async Task<SnapshotOrderAuditDetailOutDto> GetRedPackGuiderAuditDetailAsync(string id)
        => await _redPackApplication.GetRedPackGuiderAuditDetailAsync(id);

    /// <summary>
    /// 网格员红包审核通过或拒绝
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("audit/guider")]
    public async Task UpdateRedPackGuiderAuditAsync([FromBody] UpdateRedPackGuiderAuditInDto dto)
        => await _redPackApplication.AuditRedPackGuiderAuditAsync(dto);

    /// <summary>
    /// 获取审核详情
    /// </summary>
    /// <param name="id">工单Id</param>
    /// <returns></returns>
    [HttpGet("audit/{id}")]
    public async Task<SnapshotOrderAuditDetailOutDto> GetRedPackAuditDetailAsync(string id)
        => await _redPackApplication.GetRedPackAuditDetailAsync(id);

    /// <summary>
    /// 获取特提(退回)参数
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpGet("audit/back/{id}")]
    public async Task<GetAuditBackBaseDataOutDto> GetAuditBackBaseDataAsync(string id)
        => await _redPackApplication.GetAuditBackBaseDataAsync(id);

    /// <summary>
    /// 添加备注
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("audit/remark")]
    public async Task UpdateRedPackAuditRemarkAsync([FromBody] UpdateRedPackAuditRemarkInDto dto)
        => await _redPackApplication.UpdateRedPackAuditRemarkAsync(dto);

    /// <summary>
    /// 添加备注页面基础信息
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("audit/remark/basedata")]
    public async Task<Dictionary<string, object>> UpdateRedPackAuditRemarkBaseDataAsync()
    {
        return new Dictionary<string, object>()
        {
            { "failCase", EnumExts.GetDescriptions<ERedPackPickupFailCase>() }
        };
    }

    /// <summary>
    /// 添加补充发放信息页面基础信息
    /// </summary>
    /// <returns></returns>
    [HttpGet("record/basedata")]
    public async Task<Dictionary<string, object>> UpdateRedPackRecordBaseDataAsync()
    { 
        return new Dictionary<string, object>()
        {
            { "replenishType", _sysDic.SnapshotReplenishType }
        };
    }

    /// <summary>
    /// 添加补充发放信息
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("record")]
    public async Task UpdateRedPackRecordAsync([FromBody] UpdateRedPackRecordInDto dto)
        => await _redPackApplication.UpdateRedPackRecordAsync(dto);

    /// <summary>
    /// 补充发放集合基础信息
    /// </summary>
    [HttpGet("record/supplement/basedata")]
    public async Task<Dictionary<string, object>> GetRedPackRecordSupplementBaseDataAsync()
    {
        return new Dictionary<string, object>()
        {
            { "industry", await _industryRepository.GetDataBaseAsync() }
        };
    }

    /// <summary>
    /// 补充发放集合
    /// </summary>
    [HttpGet("record/supplement")]
    public async Task<PagedDto<SnapshotRedPackRecordSupplementItemsOutDto>> GetRedPackRecordSupplementItemsAsync([FromQuery] SnapshotRedPackRecordSupplementItemsInDto dto)
        => (await _redPackApplication.GetRedPackRecordSupplementItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 红包审核通过或拒绝
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("audit")]
    public async Task UpdateRedPackAuditAsync([FromBody] UpdateRedPackAuditInDto dto)
        => await _redPackApplication.AuditRedPackAuditAsync(dto);

    /// <summary>
    /// 获取审核短信模板
    /// </summary>
    /// <returns></returns>
    [HttpGet("audit/sms_template")]
    public async Task<IList<GetRedPackAuditSMSTemplateOutDto>> GetRedPackAuditSMSTemplateAsync([FromQuery]GetRedPackAuditSMSTemplateInDto dto)
        => await _redPackApplication.GetRedPackAuditSMSTemplateAsync(dto);

    /// <summary>
    /// 红包审批页面基础数据
    /// </summary>
    /// <returns></returns>
    [HttpGet("audit/basedata")]
    public async Task<Dictionary<string, object>> GetAuditBaseDataAsync()
    {
        var industry = await _industryRepository.GetDataBaseAsync();

        var configAmount = await _industryRepository.Queryable()
            .Select(m => m.CitizenReadPackAmount)
            .Distinct()
            .ToListAsync();
        var sms = await _snapshotSMSTemplateRepository.Queryable()
            .ToListAsync();
        return new Dictionary<string, object>
        {
            { "industry", industry },
            { "configAmount",  configAmount },
            { "status", EnumExts.GetDescriptions<ERedPackAuditStatus>() }
        };
    }
    #endregion

    #region 红包发放记录

    /// <summary>
    /// 市民红包发放记录
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("record")]
    public async Task<PagedDto<SnapshotRedPackRecordItemsOutDto>> GetRedPackRecordItemsAsync([FromQuery] SnapshotRedPackRecordItemsInDto dto)
        => (await _redPackApplication.GetRedPackRecordItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 网格员红包发放记录
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("record/guider")]
    public async Task<PagedDto<SnapshotRedPackRecordItemsGuiderOutDto>> GetRedPackRecordGuiderItemsAsync([FromQuery] SnapshotRedPackRecordItemsGuiderInDto dto)
        => (await _redPackApplication.GetRedPackRecordGuiderItemsAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 红包发放明细基础数据
    /// </summary>
    /// <returns></returns>
    [HttpGet("record/send/database")]
    public async Task<Dictionary<string, object>> GetRedPackRecordSendBaseDataAsync()
    {
        return new Dictionary<string, object>()
        {
            { "industry", await _industryRepository.Queryable()
            .Select(m => new { m.Id, m.Name })
            .ToListAsync()},
            { "sendStatus", EnumExts.GetDescriptions<EReadPackSendStatus>().Where(m => m.Key != 2).ToList() },
            { "userType", EnumExts.GetDescriptions<EReadPackUserType>() }
        };
    }

    /// <summary>
    /// 红包发放明细
    /// </summary>
    /// <returns></returns>
    [HttpGet("record/send")]
    public async Task<PagedDto<SnapshotRedPackRecordSendOutDto>> GetRedPackRecordDetailAsync([FromQuery] SnapshotRedPackRecordSendInDto dto)
        => (await _redPackApplication.GetRedPackRecordDetailAsync(dto).ToPagedListAsync(dto)).ToPaged();

    /// <summary>
    /// 批量发送红包
    /// </summary>
    /// <param name="ids"></param>
    /// <returns></returns>
    [HttpPut("record/send_bath")]
    public async Task<string> BathSendRedPackAsync([FromBody] List<string> ids)
    {
        return "ok";
    }
    #endregion
}
